@@ -141,6 +141,7 @@
 //! - [arrayvec](https://crates.io/crates/arrayvec), vector with fixed capacity.
 //!
 //! # Changelog
+//! - v0.3.1 - Implement `From<NotEnoughSpaceError>` and `From<MalformedInputError>` for `String`.
 //! - v0.3.0 - Breaking API changes:
 //!   - Change type parameter to const buffer size. Example: `FixedBuf<1024>`.
 //!   - Remove `new` arg.
@@ -260,6 +261,11 @@ impl From<NotEnoughSpaceError> for std::io::Error {
         )
     }
 }
+impl From<NotEnoughSpaceError> for String {
+    fn from(_: NotEnoughSpaceError) -> Self {
+        "not enough space in buffer".to_string()
+    }
+}
 
 #[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
 pub struct MalformedInputError(String);
@@ -270,7 +276,15 @@ impl MalformedInputError {
 }
 impl From<MalformedInputError> for std::io::Error {
     fn from(e: MalformedInputError) -> Self {
-        std::io::Error::new(std::io::ErrorKind::InvalidData, format!("{:?}", e))
+        std::io::Error::new(
+            std::io::ErrorKind::InvalidData,
+            format!("malformed input: {}", e.0),
+        )
+    }
+}
+impl From<MalformedInputError> for String {
+    fn from(e: MalformedInputError) -> Self {
+        format!("malformed input: {}", e.0)
     }
 }
 
@@ -438,9 +452,7 @@ impl<const SIZE: usize> FixedBuf<SIZE> {
     /// Panics if the buffer does not contain enough bytes.
     pub fn read_bytes(&mut self, num_bytes: usize) -> &[u8] {
         let new_read_index = self.read_index + num_bytes;
-        if new_read_index > self.write_index {
-            panic!("read would underflow");
-        }
+        assert!(new_read_index <= self.write_index, "read would underflow");
         let old_read_index = self.read_index;
         // We update `read_index` after any possible panic.
         // This keeps the struct consistent even when a panic happens.
@@ -612,14 +624,14 @@ impl<const SIZE: usize> FixedBuf<SIZE> {
         &mut self,
         reader: &mut R,
     ) -> Result<usize, std::io::Error> {
-        let mut writable = self.writable();
+        let writable = self.writable();
         if writable.is_empty() {
             return Err(std::io::Error::new(
                 std::io::ErrorKind::InvalidData,
                 "no empty space in buffer",
             ));
         };
-        let num_read = reader.read(&mut writable)?;
+        let num_read = reader.read(writable)?;
         self.wrote(num_read);
         Ok(num_read)
     }
@@ -721,9 +733,10 @@ impl<const SIZE: usize> FixedBuf<SIZE> {
             return;
         }
         let new_write_index = self.write_index + num_bytes;
-        if new_write_index > self.mem.as_mut().len() {
-            panic!("write would overflow");
-        }
+        assert!(
+            new_write_index <= self.mem.as_mut().len(),
+            "write would overflow"
+        );
         self.write_index = new_write_index;
     }
 
@@ -971,6 +984,26 @@ mod tests {
         deframe_line(data)
     }
 
+    #[test]
+    fn test_not_enough_space_error() {
+        let e = NotEnoughSpaceError {};
+        let io_error: std::io::Error = e.into();
+        assert_eq!(std::io::ErrorKind::InvalidData, io_error.kind());
+        assert_eq!("not enough space in buffer", &io_error.to_string());
+        let string: String = e.into();
+        assert_eq!("not enough space in buffer", &string);
+    }
+
+    #[test]
+    fn test_malformed_input_error() {
+        let e = MalformedInputError("err1".to_string());
+        let io_error: std::io::Error = e.clone().into();
+        assert_eq!(std::io::ErrorKind::InvalidData, io_error.kind());
+        assert_eq!("malformed input: err1", &io_error.to_string());
+        let string: String = e.into();
+        assert_eq!("malformed input: err1", &string);
+    }
+
     #[test]
     fn test_static_fixed_buf() {
         let mut buf = STATIC_FIXED_BUF.lock().unwrap();
